import {
    baseUrl,
    appIdStorageKey,
    baseUrlKey,
    userInfoStorageKey,
    systemInfoKey
} from './urlStore';
import store from './store.js';
import tools from './sign';

export const http = (url, data = {}, method = 'Get', hasToast = true) => {
    method = method.toUpperCase();
    let user = store.get(userInfoStorageKey) || store.get('user');
    let header = {
        token: (user && user.token) ? user.token : '',
        'content-type': 'application/x-www-form-urlencoded',
        appCode: 'ZXBLE'
    }
    if(store.get(systemInfoKey)){
        let systemInfo = store.get(systemInfoKey);
        data = Object.assign({}, data, {
            CLIENT: 'miniapp_zmwl_sdk',
            platform: systemInfo.platform,
            deviceModel: systemInfo.model,
            systemVersion: systemInfo.system,
        })
    }
    let newBaseUrl = store.get(baseUrlKey) || '';
    return new Promise((resolve, reject) => {
        wx.request({
            url: (newBaseUrl || baseUrl) + url,
            data,
            method,
            header,
            complete: async (res) => {
                if(res.statusCode==200 && res.data.success){
                    resolve(res.data.data);
                }else if(res.statusCode==401){
                    wx.showToast({
                        title: '登录信息已过期，请重新登录',
                        icon: 'none'
                    })
                    return reject(res);
                }else{
                    if(hasToast){
                        wx.showToast({
                            title: res.data.message || res.errMsg || "网络连接失败",
                            icon: 'none',
                        })
                    }
                    return reject(res);
                }
            },
        });
    })
}
